@@ -81,8 +81,9 @@ class DisconnectedEvent(PipelineEvent):
     pass
 
 
-# class NewSasTokenRequired(PipelineEvent):
-#     """
-#     A PipelineEvent object indicating that a new SasToken must be provided.
-#     """
-#     pass
+class NewSasTokenRequiredEvent(PipelineEvent):
+    """
+    A PipelineEvent object indicating that a new SasToken must be provided.
+    """
+
+    pass
