@@ -89,7 +89,7 @@ class PipelineOperation(object):
 
     @pipeline_thread.runs_on_pipeline_thread
     def complete(self, error=None):
-        """ Complete the operation, and trigger all callbacks in LIFO order.
+        """Complete the operation, and trigger all callbacks in LIFO order.
 
         The operation is completed successfully be default, or completed unsucessfully if an error
         is provided.
@@ -254,10 +254,6 @@ class ReauthorizeConnectionOperation(PipelineOperation):
     Even though this is an base operation, it will most likely be handled by a more specific stage (such as an IoTHub or MQTT stage).
     """
 
-    def __init__(self, callback):
-        self.watchdog_timer = None
-        super(ReauthorizeConnectionOperation, self).__init__(callback)
-
     pass
 
 
@@ -270,7 +266,9 @@ class DisconnectOperation(PipelineOperation):
     Even though this is an base operation, it will most likely be handled by a more specific stage (such as an IoTHub or MQTT stage).
     """
 
-    pass
+    def __init__(self, callback):
+        self.hard = True  # Indicates if this is a "hard" disconnect that kills in-flight ops
+        super(DisconnectOperation, self).__init__(callback)
 
 
 class EnableFeatureOperation(PipelineOperation):
